# coding: utf-8

from aio_client.provider.api import PostProviderRequest, push_request
from concentrator.smev3 import kinder_conc
from concentrator.smev3.base.constants import CODE_ERROR, CODE_ERROR_COMMENT
from concentrator.smev3.base.utils import (GETAPP_SERVICE_TYPE,
                                           export_response, gender_mapping,
                                           gender_mapping_child,
                                           get_contingent_param_delegate,
                                           get_declaration_units,
                                           get_dul_delegate_type,
                                           get_oktmo_region, get_privileges,
                                           type_mapping)
from concentrator.webservice.helpers import get_delegate
from kinder.core.utils.fias import name_from_guid
from kinder.webservice.api.declaration import get_decl_by_client_id
from kinder.webservice.api.exceptions import ApiException


def get_application_helper(*args, **kwargs):
    """Разбор входящего пакета и отправка ответа
    :param message:
    :return:
    """
    assert 'message' in kwargs
    message = kwargs.get('message')

    declaration, order_id = get_params(message['body'])
    response = get_application_response(declaration, order_id)

    response = push_request(PostProviderRequest(
        origin_message_id=message["origin_message_id"],
        body=export_response(response),
        message_type=message["message_type"],
        attachments='',
        content_failure_code=None,
        replay_to=message["replay_to"],
        is_test_message = message["is_test_message"]
    ))

    return response


def get_params(body):
    """Возвразает заявление в системе, и идениифкатор этого заявлени яиз запроса
    :param body: тело запроса
    :return: список  declaration, order_id
    """
    request_body = kinder_conc.parseString(body)
    order_id = request_body.GetApplicationRequest.orderId
    try:
        declaration = get_decl_by_client_id(order_id)
    except ApiException:
        declaration = None
    return declaration, order_id


def get_application_response(declaration, order_id):
    """Подготовка ответа для метода GetApplication

    :param declaration: Заявление
    :return: xml
    """
    if not declaration:
        return kinder_conc.FormDataResponseType(
            oktmo=get_oktmo_region(),
            changeOrderInfo=kinder_conc.changeOrderInfoType(
                orderId=kinder_conc.orderIdType(order_id),
                statusCode=kinder_conc.statusCodeType(CODE_ERROR),
                comment=CODE_ERROR_COMMENT,
            )
        )

    decl_units = get_declaration_units(declaration)
    child = declaration.children
    delegate = get_delegate(declaration)
    if not delegate:
        raise ApiException(u'Не найден представитель')
    citizenship, citizenship_country, birthplace = get_contingent_param_delegate(delegate)

    delegate_doc = kinder_conc.PersonIdentityDocInfoType(
        IdentityDocName=kinder_conc.DataElementType(
            code=delegate.dul_type_id,
            valueOf_=get_dul_delegate_type(delegate)),
        IdentityDocSeries=delegate.dul_series,
        IdentityDocNumber=delegate.dul_number,
        IdentityDocIssueDate=delegate.dul_date,
        IdentityDocIssueCode=delegate.dul_place,
        IdentityDocIssued=delegate.dul_issued_by,
        Citizenship=citizenship,
        BirthPlace=birthplace
    )
    type_code, type_name = type_mapping(delegate)

    delegate_info = kinder_conc.PersonInfoType(
        PersonSurname=delegate.surname,
        PersonName=delegate.firstname,
        PersonMiddleName=delegate.patronymic,
        PersonBirthDate=delegate.date_of_birth,
        PersonSex=gender_mapping(delegate),
        PersonSNILS=delegate.snils,
        PersonPhone=delegate.phones,
        PersonEmail=delegate.email,
        PersonIdentityDocInfo=delegate_doc,
        PersonType=kinder_conc.DataElementType(
            code=type_code,
            valueOf_=type_name
        )
    )

    child_doc = kinder_conc.ChildBirthDocRFType(
        ChildBirthDocSeries=child.dul_series,
        ChildBirthDocNumber=child.dul_number,
        ChildBirthDocActNumber=child.zags_act_number ,
        ChildBirthDocIssueDate=child.dul_date,
        ChildBirthDocIssued=child.zags_act_place,
        ChildBirthPlace=child.birthplace,
    )

    child_info = kinder_conc.ChildInfoType(
        ChildSurname=child.surname,
        ChildName=child.firstname,
        ChildMiddleName=child.patronymic,
        ChildSex=gender_mapping_child(child),
        ChildBirthDate=child.date_of_birth,
        ChildSNILS=child.snils,
        ChildBirthDocRF=child_doc
    )

    address_info = kinder_conc.AddressType(
        FullAddress=child.reg_address_full,
        Region=kinder_conc.DataElementType(
            code=child.reg_address_place,
            valueOf_=name_from_guid(child.reg_address_place)),
        Street=kinder_conc.DataElementType(
            code=child.reg_address_street,
            valueOf_=name_from_guid(child.reg_address_street)),
        House=kinder_conc.DataElementType(
            code=child.reg_address_house_guid,
            valueOf_=child.reg_address_house),
        Building1=child.reg_address_corps,
        Apartment=child.reg_address_flat,
    )

    edu_org_info = kinder_conc.EduOrganizationsType(
        AllowOfferOther=declaration.offer_other)
    for (unit_name, unit_id, ord) in decl_units:
        edu_org_info.add_EduOrganization(kinder_conc.EduOrganizationType(
            code=unit_id,
            priority=ord == '1',
            valueOf_=unit_name

        ))

    benefits_info = kinder_conc.BenefitsInfoType()
    for param in get_privileges(declaration):
        id, name, doc_type_id, doc_type_name, doc_series, doc_number = param
        benefits_info.add_BenefitInfo(
            kinder_conc.BenefitInfoType(
                BenefitCategory = kinder_conc.DataElementType(
                    code=id,
                    valueOf_=name,
                ),
                BenefitDocInfo= kinder_conc.DocInfoType(
                    DocName=kinder_conc.DataElementType(
                        code=doc_type_id,
                        valueOf_=doc_type_name
                    ),
                    DocSeries=doc_series,
                    DocNumber=doc_number
                )
            ))
    app_type = kinder_conc.ApplicationType(
        orderId=int(declaration.client_id),
        ServicesType=GETAPP_SERVICE_TYPE,
        PersonInfo=delegate_info,
        ChildInfo=child_info,
        Address=address_info,
        EduOrganizations=edu_org_info,
        EntryDate=declaration.desired_date,
        AdaptationProgram=None,
        ScheduleType=kinder_conc.DataElementType(
            code=getattr(declaration, 'work_type__id', None),
            valueOf_=getattr(declaration,'work_type__name', None)
        ),
        BenefitsInfo=benefits_info
    )
    response = kinder_conc.FormDataResponseType(
        oktmo=get_oktmo_region(),
        GetApplicationResponse=kinder_conc.GetApplicationResponseType(
            Application=app_type))

    return response
